package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that verifies, for every controllers of a class, that they do not
 * have a null pointer exception.
 * 
 * @author rdelamare
 * 
 */
public class NPEConstructorChecker extends ClassChecker {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(NPEConstructorChecker.class);

	public NPEConstructorChecker(final IInstanceProvider provider,
			final Class<?> clazz) {
		super(provider, clazz);
	}

	public void check() {
		if (clazz != null)
			for (final Constructor<?> constructor : clazz
					.getDeclaredConstructors()) {
				if (Modifier.isPublic(constructor.getModifiers()))
					try {
						constructor.newInstance(Utils
								.generateDefaultValuesParameters(constructor
										.getParameterTypes()));
					} catch (final IllegalArgumentException e) {
						LOGGER.error(
								"Error : illegal argument exception while invoking constructor for the {} class",
								clazz.getCanonicalName());
					} catch (final InstantiationException e) {
						LOGGER.error(
								"Error : instantiation exception while invoking constructor for the {} class",
								clazz.getCanonicalName());
					} catch (final IllegalAccessException e) {
						LOGGER.error(
								"Error : illegal access exception while invoking constructor for the {} class",
								clazz.getCanonicalName());
					} catch (final InvocationTargetException e) {
						if (e.getCause() instanceof NullPointerException)
							throw new EasycoverageAssertionError(
									"Constructor threw NullPointerException with default values.",
									e.getCause());
					}
			}
	}

	@Override
	public boolean shouldCheck() {
		if (!super.shouldCheck())
			return false;
		for (final Constructor<?> constructor : clazz.getDeclaredConstructors()) {
			if (Modifier.isPublic(constructor.getModifiers())) {
				return true;
			}
		}
		return false;
	}

}
