package com.tocea.easycoverage.framework.providers;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * Default instance provider. It will look for the constructor with the least
 * number of parameters and try to invoke it to create an instance.
 * 
 * @author Romain Delamare
 * 
 */
public class DefaultInstanceProvider implements IInstanceProvider {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(DefaultInstanceProvider.class);

	public <T> T getInstance(final Class<T> clazz) {
		if (clazz == null)
			return null;
		if (clazz.isPrimitive())
			return getPrimitive(clazz);
		final Constructor<T> constructor = getConstructor(clazz);
		T t = null;
		if (constructor != null)
			try {
				t = constructor.newInstance(Utils
						.generateDefaultValuesParameters(constructor
								.getParameterTypes()));
			} catch (final IllegalArgumentException e) {
				LOGGER.error(
						"Error, could not instantiate {} : illegal argument exception",
						clazz.getCanonicalName());
			} catch (final InstantiationException e) {
				LOGGER.error(
						"Error, could not instantiate {} : instantiation exception",
						clazz.getCanonicalName());
			} catch (final IllegalAccessException e) {
				LOGGER.error(
						"Error, could not instantiate {} : illegal access exception",
						clazz.getCanonicalName());
			} catch (final InvocationTargetException e) {
				throw new RuntimeException(e.getCause());
			}
		return t;
	}

	public <T> boolean canProvide(final Class<T> clazz) {
		return clazz != null
				&& (clazz.isPrimitive() || !Modifier.isAbstract(clazz
						.getModifiers()) && getConstructor(clazz) != null);
	}

	@SuppressWarnings("unchecked")
	private static <T> Constructor<T> getConstructor(final Class<T> clazz) {
		Constructor<T> constructor = null;
		for (final Constructor<?> c : clazz.getConstructors()) {
			if (constructor == null
					|| c.getParameterTypes().length < constructor
							.getParameterTypes().length)
				constructor = (Constructor<T>) c;
		}
		return constructor;
	}

	@SuppressWarnings("unchecked")
	private static <T> T getPrimitive(final Class<T> clazz) {
		T t;
		if (int.class.equals(clazz) || Integer.class.equals(clazz)) {
			t = (T) new Integer(0);
		}
		else if (short.class.equals(clazz) || Short.class.equals(clazz)) {
			t = (T) new Short((short) 0);
		}
		else if (byte.class.equals(clazz) || Byte.class.equals(clazz)) {
			t = (T) new Byte((byte) 0);
		}
		else if (long.class.equals(clazz) || Long.class.equals(clazz)) {
			t = (T) new Long(0l);
		}
		else if (float.class.equals(clazz) || Float.class.equals(clazz)) {
			t = (T) new Float(0f);
		}
		else if (double.class.equals(clazz) || Double.class.equals(clazz)) {
			t = (T) new Double(0d);
		}
		else if (char.class.equals(clazz) || Character.class.equals(clazz)) {
			t = (T) new Character('\u0000');
		}
		else if (boolean.class.equals(clazz) || Boolean.class.equals(clazz)) {
			t = (T) new Boolean(false);
		}
		else {
			t = null;
		}
		return t;
	}

}
